using System;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;

namespace Sturm.CollectionWrapping {
	/// <summary>
	/// This wrapper implements all the same interfaces that the BindingList&lt;T&gt; class
	/// also implements - the combination of IBindingList and IList&lt;T&gt;, which 
	/// BindingList&lt;T&gt; inherits from Collection&lt;T&gt;, is unique, and
	/// there are also some other interfaces.
	/// </summary>
	/// <typeparam name="T"></typeparam>
	public class TypedListWrapperBindingListT<T> : BindingList<T>, IBindingList, IList<T>, ICancelAddNew, IRaiseItemChangedEvents, ITypedList {
		public TypedListWrapperBindingListT( ) {

		}

		public TypedListWrapperBindingListT(BindingList<T> wrappedBindingListT) {
			this.wrappedBindingListT = wrappedBindingListT;
		}

		public TypedListWrapperBindingListT(BindingList<T> wrappedBindingListT, PropertyDescriptorCollection propertyDescriptorCollection) {
			this.propertyDescriptorCollection = propertyDescriptorCollection;
			this.wrappedBindingListT = wrappedBindingListT;
		}

		public TypedListWrapperBindingListT(PropertyDescriptorCollection propertyDescriptorCollection) {
			this.propertyDescriptorCollection = propertyDescriptorCollection;
		}

		private BindingList<T> wrappedBindingListT;
		public BindingList<T> WrappedBindingListT {
			get {
				return wrappedBindingListT;
			}
			set {
				wrappedBindingListT = value;
			}
		}

		private PropertyDescriptorCollection propertyDescriptorCollection;
		public PropertyDescriptorCollection PropertyDescriptorCollection {
			get {
				return propertyDescriptorCollection;
			}
			set {
				propertyDescriptorCollection = value;
			}
		}

		private string listName = "Default";
		public string ListName {
			get {
				return listName;
			}
			set {
				listName = value;
			}
		}

		#region ITypedList Members

		public virtual PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[] listAccessors) {
			return propertyDescriptorCollection;
		}

		public virtual string GetListName(PropertyDescriptor[] listAccessors) {
			return listName;
		}

		#endregion

		#region IBindingList Members

		void IBindingList.AddIndex(PropertyDescriptor property) {
			((IBindingList) wrappedBindingListT).AddIndex(property);
		}

		object IBindingList.AddNew( ) {
			return ((IBindingList) wrappedBindingListT).AddNew( );
		}

		bool IBindingList.AllowEdit {
			get { return ((IBindingList) wrappedBindingListT).AllowEdit; }
		}

		bool IBindingList.AllowNew {
			get { return ((IBindingList) wrappedBindingListT).AllowNew; }
		}

		bool IBindingList.AllowRemove {
			get { return ((IBindingList) wrappedBindingListT).AllowRemove; }
		}

		void IBindingList.ApplySort(PropertyDescriptor property, ListSortDirection direction) {
			((IBindingList) wrappedBindingListT).ApplySort(property, direction);
		}

		int IBindingList.Find(PropertyDescriptor property, object key) {
			return ((IBindingList) wrappedBindingListT).Find(property, key);
		}

		bool IBindingList.IsSorted {
			get { return ((IBindingList) wrappedBindingListT).IsSorted; }
		}

		event ListChangedEventHandler IBindingList.ListChanged {
			add { ((IBindingList) wrappedBindingListT).ListChanged += value; }
			remove { ((IBindingList) wrappedBindingListT).ListChanged -= value; }
		}

		void IBindingList.RemoveIndex(PropertyDescriptor property) {
			((IBindingList) wrappedBindingListT).RemoveIndex(property);
		}

		void IBindingList.RemoveSort( ) {
			((IBindingList) wrappedBindingListT).RemoveSort( );
		}

		ListSortDirection IBindingList.SortDirection {
			get { return ((IBindingList) wrappedBindingListT).SortDirection; }
		}

		PropertyDescriptor IBindingList.SortProperty {
			get { return ((IBindingList) wrappedBindingListT).SortProperty; }
		}

		bool IBindingList.SupportsChangeNotification {
			get { return ((IBindingList) wrappedBindingListT).SupportsChangeNotification; }
		}

		bool IBindingList.SupportsSearching {
			get { return ((IBindingList) wrappedBindingListT).SupportsSearching; }
		}

		bool IBindingList.SupportsSorting {
			get { return ((IBindingList) wrappedBindingListT).SupportsSorting; }
		}

		#endregion

		#region ICancelAddNew Members

		void ICancelAddNew.CancelNew(int itemIndex) {
			wrappedBindingListT.CancelNew(itemIndex);
		}

		void ICancelAddNew.EndNew(int itemIndex) {
			wrappedBindingListT.EndNew(itemIndex);
		}

		#endregion

		#region IRaiseItemChangedEvents Members

		bool IRaiseItemChangedEvents.RaisesItemChangedEvents {
			get { return wrappedBindingListT.RaiseListChangedEvents; }
		}

		#endregion

		#region IList Members

		int IList.Add(object value) {
			return ((IList) wrappedBindingListT).Add(value);
		}

		void IList.Clear( ) {
			((IList) wrappedBindingListT).Clear( );
		}

		bool IList.Contains(object value) {
			return ((IList) wrappedBindingListT).Contains(value);
		}

		int IList.IndexOf(object value) {
			return ((IList) wrappedBindingListT).IndexOf(value);
		}

		void IList.Insert(int index, object value) {
			((IList) wrappedBindingListT).Insert(index, value);
		}

		bool IList.IsFixedSize {
			get { return ((IList) wrappedBindingListT).IsFixedSize; }
		}

		bool IList.IsReadOnly {
			get { return ((IList) wrappedBindingListT).IsReadOnly; }
		}

		void IList.Remove(object value) {
			((IList) wrappedBindingListT).Remove(value);
		}

		void IList.RemoveAt(int index) {
			((IList) wrappedBindingListT).RemoveAt(index);
		}

		object IList.this[int index] {
			get {
				return ((IList) wrappedBindingListT)[index];
			}
			set {
				((IList) wrappedBindingListT)[index] = value;
			}
		}

		#endregion

		#region ICollection Members

		void ICollection.CopyTo(Array array, int index) {
			((ICollection) wrappedBindingListT).CopyTo(array, index);
		}

		int ICollection.Count {
			get { return ((ICollection) wrappedBindingListT).Count; }
		}

		bool ICollection.IsSynchronized {
			get { return ((ICollection) wrappedBindingListT).IsSynchronized; }
		}

		object ICollection.SyncRoot {
			get { return ((ICollection) wrappedBindingListT).SyncRoot; }
		}

		#endregion

		#region IEnumerable Members

		IEnumerator IEnumerable.GetEnumerator( ) {
			return ((IEnumerable) wrappedBindingListT).GetEnumerator( );
		}

		#endregion

		#region IList<T> Members

		int IList<T>.IndexOf(T item) {
			return ((IList<T>) wrappedBindingListT).IndexOf(item);
		}

		void IList<T>.Insert(int index, T item) {
			((IList<T>) wrappedBindingListT).Insert(index, item);
		}

		void IList<T>.RemoveAt(int index) {
			((IList<T>) wrappedBindingListT).RemoveAt(index);
		}

		T IList<T>.this[int index] {
			get {
				return ((IList<T>) wrappedBindingListT)[index];
			}
			set {
				((IList<T>) wrappedBindingListT)[index] = value;
			}
		}

		#endregion

		#region ICollection<T> Members

		void ICollection<T>.Add(T item) {
			((ICollection<T>) wrappedBindingListT).Add(item);
		}

		void ICollection<T>.Clear( ) {
			((ICollection<T>) wrappedBindingListT).Clear( );
		}

		bool ICollection<T>.Contains(T item) {
			return ((ICollection<T>) wrappedBindingListT).Contains(item);
		}

		void ICollection<T>.CopyTo(T[] array, int arrayIndex) {
			((ICollection<T>) wrappedBindingListT).CopyTo(array, arrayIndex);
		}

		int ICollection<T>.Count {
			get { return ((ICollection<T>) wrappedBindingListT).Count; }
		}

		bool ICollection<T>.IsReadOnly {
			get { return ((ICollection<T>) wrappedBindingListT).IsReadOnly; }
		}

		bool ICollection<T>.Remove(T item) {
			return ((ICollection<T>) wrappedBindingListT).Remove(item);
		}

		#endregion

		#region IEnumerable<T> Members

		IEnumerator<T> IEnumerable<T>.GetEnumerator( ) {
			return ((IEnumerable<T>) wrappedBindingListT).GetEnumerator( );
		}

		#endregion
	}
}
